function [M, WIRE_OUT] = tutorial_peaking_filter_process(M, WIRE_IN)
% [M, WIRE_OUT] = tutorial_peaking_filter_process(M, WIRE_IN)
% Processing function for the tutorial_peaking_filter_module.m.  Arguments:
%   M - @awe_module object.
%   WIRE_IN - cell array of input wires.
% The function returns the updated module object and a cell array of
% output wires.

% Copyright 2020.  DSP Concepts, Inc.  All Rights Reserved.
% Author:  Paul Beckmann

IN = WIRE_IN{1};
[blockSize, numChannels] = size(IN);

% Smoothly update the coefficients
M.currentCoeffs = M.currentCoeffs * (1 - M.smoothingCoeff) + M.coeffs * M.smoothingCoeff;

b0 = M.currentCoeffs(1);
b1 = M.currentCoeffs(2);
b2 = M.currentCoeffs(3);
a1 = -M.currentCoeffs(4);
a2 = -M.currentCoeffs(5);

wNm1 = M.state(1, :);
wNm2 = M.state(2, :);

OUT = zeros(blockSize, numChannels);

for i = 1:blockSize
    wN = a1*wNm1 + a2*wNm2 + IN(i, :);
    OUT(i, :)= b0*wN + b1*wNm1 + b2*wNm2;
    wNm2 = wNm1;
    wNm1 = wN;
end

M.state = [wNm1; wNm2];

WIRE_OUT{1} = OUT;

return;
